@@ -32,7 +32,7 @@
 
 /* conn_notice_callback - process notices */
 
-void
+static void
 conn_notice_callback(void *args, const char *message)
 {
     connectionObject *self = (connectionObject *)args;
@@ -40,33 +40,44 @@ conn_notice_callback(void *args, const char *message)
     Dprintf("conn_notice_callback: %s", message);
 
     /* unfortunately the old protocl return COPY FROM errors only as notices,
-       so we need to filter them looking for such errors */
-    if (strncmp(message, "ERROR", 5) == 0)
+       so we need to filter them looking for such errors (but we do it
+       only if the protocol if <3, else we don't need that */
+
+    if (self->protocol < 3 && strncmp(message, "ERROR", 5) == 0)
         pq_set_critical(self, message);
-    else
-        PyList_Append(self->notice_list, PyString_FromString(message));
+    else {
+        PyObject *msg = PyString_FromString(message);
+
+        PyList_Append(self->notice_list, msg);
+        Py_DECREF(msg);
+
+        /* Remove the oldest item if the queue is getting too long. */
+        if (PyList_GET_SIZE(self->notice_list) > CONN_NOTICES_LIMIT)
+            PySequence_DelItem(self->notice_list, 0);
+    }
 }
 
-/* conn_connect - execute a connection to the dataabase */
+/* conn_connect - execute a connection to the database */
 
 int
 conn_connect(connectionObject *self)
 {
     PGconn *pgconn;
     PGresult *pgres;
-    char *data, *tmp;
+    const char *data, *tmp;
+    const char *scs;    /* standard-conforming strings */
     size_t i;
 
     /* we need the initial date style to be ISO, for typecasters; if the user
        later change it, she must know what she's doing... */
-    const char *datestyle = "SET DATESTYLE TO 'ISO'";
-    const char *encoding  = "SHOW client_encoding";
-    const char *isolevel  = "SHOW default_transaction_isolation";
+    static const char datestyle[] = "SET DATESTYLE TO 'ISO'";
+    static const char encoding[]  = "SHOW client_encoding";
+    static const char isolevel[]  = "SHOW default_transaction_isolation";
 
-    const char *lvl1a = "read uncommitted";
-    const char *lvl1b = "read committed";
-    const char *lvl2a = "repeatable read";
-    const char *lvl2b = "serializable";
+    static const char lvl1a[] = "read uncommitted";
+    static const char lvl1b[] = "read committed";
+    static const char lvl2a[] = "repeatable read";
+    static const char lvl2b[] = "serializable";
 
     Py_BEGIN_ALLOW_THREADS;
     pgconn = PQconnectdb(self->dsn);
@@ -90,6 +101,34 @@ conn_connect(connectionObject *self)
 
     PQsetNoticeProcessor(pgconn, conn_notice_callback, (void*)self);
 
+    /*
+     * The presence of the 'standard_conforming_strings' parameter
+     * means that the server _accepts_ the E'' quote.
+     *
+     * If the paramer is off, the PQescapeByteaConn returns
+     * backslash escaped strings (e.g. '\001' -> "\\001"),
+     * so the E'' quotes are required to avoid warnings
+     * if 'escape_string_warning' is set.
+     *
+     * If the parameter is on, the PQescapeByteaConn returns
+     * not escaped strings (e.g. '\001' -> "\001"), relying on the
+     * fact that the '\' will pass untouched the string parser.
+     * In this case the E'' quotes are NOT to be used.
+     *
+     * The PSYCOPG_OWN_QUOTING implementation always returns escaped strings.
+     */
+    scs = PQparameterStatus(pgconn, "standard_conforming_strings");
+    Dprintf("conn_connect: server standard_conforming_strings parameter: %s",
+        scs ? scs : "unavailable");
+
+#ifndef PSYCOPG_OWN_QUOTING
+    self->equote = (scs && (0 == strcmp("off", scs)));
+#else
+    self->equote = (scs != NULL);
+#endif
+    Dprintf("conn_connect: server requires E'' quotes: %s",
+        self->equote ? "YES" : "NO");
+
     Py_BEGIN_ALLOW_THREADS;
     pgres = PQexec(pgconn, datestyle);
     Py_END_ALLOW_THREADS;
@@ -113,9 +152,9 @@ conn_connect(connectionObject *self)
         return -1;
     }
     tmp = PQgetvalue(pgres, 0, 0);
-    self->encoding = PyMem_Malloc(strlen(tmp)+1);
+    self->encoding = malloc(strlen(tmp)+1);
     if (self->encoding == NULL) {
-        /* exception already set by PyMem_Malloc() */
+        PyErr_NoMemory();
         PQfinish(pgconn);
         IFCLEARPGRES(pgres);
         return -1;
@@ -176,20 +215,29 @@ conn_close(connectionObject *self)
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&self->lock);
 
-    self->closed = 1;
+    if (self->closed == 0)
+        self->closed = 1;
 
     /* execute a forced rollback on the connection (but don't check the
        result, we're going to close the pq connection anyway */
+    if (self->pgconn && self->closed == 1) {
+        PGresult *pgres = NULL;
+        char *error = NULL;
+
+        if (pq_abort_locked(self, &pgres, &error) < 0) {
+            IFCLEARPGRES(pgres);
+            if (error)
+                free (error);
+        }
+    }
     if (self->pgconn) {
-        pq_abort(self);
         PQfinish(self->pgconn);
         Dprintf("conn_close: PQfinish called");
         self->pgconn = NULL;
-    }
-
+   }
+   
     pthread_mutex_unlock(&self->lock);
     Py_END_ALLOW_THREADS;
-
 }
 
 /* conn_commit - commit on a connection */
@@ -199,15 +247,7 @@ conn_commit(connectionObject *self)
 {
     int res;
 
-    Py_BEGIN_ALLOW_THREADS;
-    pthread_mutex_lock(&self->lock);
-
     res = pq_commit(self);
-    self->mark++;
-
-    pthread_mutex_unlock(&self->lock);
-    Py_END_ALLOW_THREADS;
-
     return res;
 }
 
@@ -218,15 +258,7 @@ conn_rollback(connectionObject *self)
 {
     int res;
 
-    Py_BEGIN_ALLOW_THREADS;
-    pthread_mutex_lock(&self->lock);
-
     res = pq_abort(self);
-    self->mark++;
-
-    pthread_mutex_unlock(&self->lock);
-    Py_END_ALLOW_THREADS;
-
     return res;
 }
 
@@ -235,6 +267,8 @@ conn_rollback(connectionObject *self)
 int
 conn_switch_isolation_level(connectionObject *self, int level)
 {
+    PGresult *pgres = NULL;
+    char *error = NULL;
     int res = 0;
 
     /* if the current isolation level is equal to the requested one don't switch */
@@ -246,25 +280,28 @@ conn_switch_isolation_level(connectionObject *self, int level)
     /* if the current isolation level is > 0 we need to abort the current
        transaction before changing; that all folks! */
     if (self->isolation_level != level && self->isolation_level > 0) {
-        res = pq_abort(self);
+        res = pq_abort_locked(self, &pgres, &error);
     }
     self->isolation_level = level;
-    self->mark++;
 
     Dprintf("conn_switch_isolation_level: switched to level %d", level);
 
     pthread_mutex_unlock(&self->lock);
     Py_END_ALLOW_THREADS;
 
+    if (res < 0)
+        pq_complete_error(self, &pgres, &error);
+
     return res;
 }
 
 /* conn_set_client_encoding - switch client encoding on connection */
 
 int
-conn_set_client_encoding(connectionObject *self, char *enc)
+conn_set_client_encoding(connectionObject *self, const char *enc)
 {
-    PGresult *pgres;
+    PGresult *pgres = NULL;
+    char *error = NULL;
     char query[48];
     int res = 0;
 
@@ -282,30 +319,25 @@ conn_set_client_encoding(connectionObject *self, char *enc)
 
     /* abort the current transaction, to set the encoding ouside of
        transactions */
-    res = pq_abort(self);
+    res = pq_abort_locked(self, &pgres, &error);
 
     if (res == 0) {
-        pgres = PQexec(self->pgconn, query);
-
-        if (pgres == NULL || PQresultStatus(pgres) != PGRES_COMMAND_OK ) {
-            res = -1;
-        }
-        else {
+        res = pq_execute_command_locked(self, query, &pgres, &error);
+        if (res == 0) {
             /* no error, we can proceeed and store the new encoding */
             if (self->encoding) free(self->encoding);
             self->encoding = strdup(enc);
+            Dprintf("conn_set_client_encoding: set encoding to %s",
+                    self->encoding);
         }
-
-        IFCLEARPGRES(pgres);
     }
 
-    Dprintf("conn_set_client_encoding: set encoding to %s", self->encoding);
 
     pthread_mutex_unlock(&self->lock);
     Py_END_ALLOW_THREADS;
 
-    if (res == -1)
-        PyErr_Format(OperationalError, "can't set encoding to %s", enc);
+    if (res < 0)
+        pq_complete_error(self, &pgres, &error);
 
     return res;
 }
