@@ -521,6 +521,7 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
     while ((v = PyIter_Next(vars)) != NULL) {
         if (_psyco_curs_execute(self, operation, v, 0) == 0) {
             Py_DECREF(v);
+            Py_XDECREF(iter);
             return NULL;
         }
         else {
@@ -1589,8 +1590,8 @@ cursor_setup(cursorObject *self, connectionObject *conn, const char *name)
             "argument 1 must be subclass of psycopg2._psycopg.connection");
         return 1;
     } */
+    Py_INCREF(conn);
     self->conn = conn;
-    Py_INCREF((PyObject*)self->conn);
 
     self->closed = 0;
     self->mark = conn->mark;
@@ -1606,6 +1607,7 @@ cursor_setup(cursorObject *self, connectionObject *conn, const char *name)
     self->string_types = NULL;
     self->binary_types = NULL;
 
+    Py_INCREF(Py_None);
     self->description = Py_None;
     Py_INCREF(Py_None);
     self->pgstatus = Py_None;
@@ -1613,11 +1615,10 @@ cursor_setup(cursorObject *self, connectionObject *conn, const char *name)
     self->tuple_factory = Py_None;
     Py_INCREF(Py_None);
     self->query = Py_None;
-    Py_INCREF(Py_None);
 
     /* default tzinfo factory */
+    Py_INCREF(pyPsycopgTzFixedOffsetTimezone);
     self->tzinfo_factory = pyPsycopgTzFixedOffsetTimezone;
-    Py_INCREF(self->tzinfo_factory);
 
     Dprintf("cursor_setup: good cursor object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
@@ -1633,15 +1634,15 @@ cursor_dealloc(PyObject* obj)
 
     if (self->name) PyMem_Free(self->name);
 
-    Py_XDECREF((PyObject*)self->conn);
-    Py_XDECREF(self->casts);
-    Py_XDECREF(self->description);
-    Py_XDECREF(self->pgstatus);
-    Py_XDECREF(self->tuple_factory);
-    Py_XDECREF(self->tzinfo_factory);
-    Py_XDECREF(self->query);
-    Py_XDECREF(self->string_types);
-    Py_XDECREF(self->binary_types);
+    Py_CLEAR(self->conn);
+    Py_CLEAR(self->casts);
+    Py_CLEAR(self->description);
+    Py_CLEAR(self->pgstatus);
+    Py_CLEAR(self->tuple_factory);
+    Py_CLEAR(self->tzinfo_factory);
+    Py_CLEAR(self->query);
+    Py_CLEAR(self->string_types);
+    Py_CLEAR(self->binary_types);
 
     IFCLEARPGRES(self->pgres);
 
@@ -1674,7 +1675,7 @@ cursor_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static void
 cursor_del(PyObject* self)
 {
-    PyObject_Del(self);
+    PyObject_GC_Del(self);
 }
 
 static PyObject *
@@ -1684,6 +1685,23 @@ cursor_repr(cursorObject *self)
         "<cursor object at %p; closed: %d>", self, self->closed);
 }
 
+static int
+cursor_traverse(cursorObject *self, visitproc visit, void *arg)
+{
+    Py_VISIT((PyObject *)self->conn);
+    Py_VISIT(self->description);
+    Py_VISIT(self->pgstatus);
+    Py_VISIT(self->casts);
+    Py_VISIT(self->caster);
+    Py_VISIT(self->copyfile);
+    Py_VISIT(self->tuple_factory);
+    Py_VISIT(self->tzinfo_factory);
+    Py_VISIT(self->query);
+    Py_VISIT(self->string_types);
+    Py_VISIT(self->binary_types);
+    return 0;
+}
+
 
 /* object type */
 
@@ -1713,10 +1731,11 @@ PyTypeObject cursorType = {
     0,          /*tp_setattro*/
     0,          /*tp_as_buffer*/
 
-    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_ITER, /*tp_flags*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_ITER |
+      Py_TPFLAGS_HAVE_GC, /*tp_flags*/
     cursorType_doc, /*tp_doc*/
 
-    0,          /*tp_traverse*/
+    (traverseproc)cursor_traverse, /*tp_traverse*/
     0,          /*tp_clear*/
 
     0,          /*tp_richcompare*/
