@@ -24,6 +24,7 @@
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
+#include <libpq-fe.h>
 
 #include "psycopg/config.h"
 
@@ -142,6 +143,8 @@ HIDDEN PyObject *psyco_GetDecimalType(void);
 HIDDEN void psyco_set_error(PyObject *exc, PyObject *curs,  const char *msg,
                             const char *pgerror, const char *pgcode);
 
+HIDDEN size_t qstring_escape(char *to, char *from, size_t len, PGconn *conn);
+
 /* Exceptions docstrings */
 #define Error_doc \
 "Base class for error exceptions."
