@@ -28,6 +28,7 @@
 #include "psycopg/psycopg.h"
 #include "psycopg/connection.h"
 #include "psycopg/cursor.h"
+#include "psycopg/lobject.h"
 #include "psycopg/typecast.h"
 #include "psycopg/microprotocols.h"
 #include "psycopg/microprotocols_proto.h"
@@ -42,27 +43,30 @@
 #ifdef HAVE_MXDATETIME
 #include <mxDateTime.h>
 #include "psycopg/adapter_mxdatetime.h"
-mxDateTimeModule_APIObject *mxDateTimeP = NULL;
+HIDDEN mxDateTimeModule_APIObject *mxDateTimeP = NULL;
 #endif
 
 /* some module-level variables, like the datetime module */
 #ifdef HAVE_PYDATETIME
 #include <datetime.h>
 #include "psycopg/adapter_datetime.h"
-PyObject *pyDateTimeModuleP = NULL;
-PyObject *pyDateTypeP = NULL;
-PyObject *pyTimeTypeP = NULL;
-PyObject *pyDateTimeTypeP = NULL;
-PyObject *pyDeltaTypeP = NULL;
+HIDDEN PyObject *pyDateTimeModuleP = NULL;
+HIDDEN PyObject *pyDateTypeP = NULL;
+HIDDEN PyObject *pyTimeTypeP = NULL;
+HIDDEN PyObject *pyDateTimeTypeP = NULL;
+HIDDEN PyObject *pyDeltaTypeP = NULL;
 #endif
 
 /* pointers to the psycopg.tz classes */
-PyObject *pyPsycopgTzModule = NULL;
-PyObject *pyPsycopgTzLOCAL = NULL;
-PyObject *pyPsycopgTzFixedOffsetTimezone = NULL;
+HIDDEN PyObject *pyPsycopgTzModule = NULL;
+HIDDEN PyObject *pyPsycopgTzLOCAL = NULL;
+HIDDEN PyObject *pyPsycopgTzFixedOffsetTimezone = NULL;
 
-PyObject *psycoEncodings = NULL;
-PyObject *decimalType = NULL;
+HIDDEN PyObject *psycoEncodings = NULL;
+
+#ifdef PSYCOPG_DEBUG
+HIDDEN int psycopg_debug_enabled = 0;
+#endif
 
 /** connect module-level function **/
 #define psyco_connect_doc \
@@ -90,39 +94,13 @@ PyObject *decimalType = NULL;
 ":rtype: `extensions.connection`"
 
 static size_t
-_psyco_connect_fill_dsn(char *dsn, char *kw, char *v, size_t i)
+_psyco_connect_fill_dsn(char *dsn, const char *kw, const char *v, size_t i)
 {
     strcpy(&dsn[i], kw); i += strlen(kw);
     strcpy(&dsn[i], v); i += strlen(v);
     return i;
 }
 
-static void
-_psyco_connect_fill_exc(connectionObject *conn)
-{
-    /* fill the connection object with the exceptions */
-    conn->exc_Error = Error;
-    Py_INCREF(Error);
-    conn->exc_Warning = Warning;
-    Py_INCREF(Warning);
-    conn->exc_InterfaceError = InterfaceError;
-    Py_INCREF(InterfaceError);
-    conn->exc_DatabaseError = DatabaseError;
-    Py_INCREF(DatabaseError);
-    conn->exc_InternalError = InternalError;
-    Py_INCREF(InternalError);
-    conn->exc_ProgrammingError = ProgrammingError;
-    Py_INCREF(ProgrammingError);
-    conn->exc_IntegrityError = IntegrityError;
-    Py_INCREF(IntegrityError);
-    conn->exc_DataError = DataError;
-    Py_INCREF(DataError);
-    conn->exc_NotSupportedError = NotSupportedError;
-    Py_INCREF(NotSupportedError);
-    conn->exc_OperationalError = OperationalError;
-    Py_INCREF(OperationalError);
-}
-
 static PyObject *
 psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
 {
@@ -131,9 +109,10 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
 
     size_t idsn=-1;
     int iport=-1;
-    char *dsn_static=NULL, *dsn_dynamic=NULL;
-    char *database=NULL, *user=NULL, *password=NULL;
-    char *host=NULL, *sslmode=NULL;
+    const char *dsn_static = NULL;
+    char *dsn_dynamic=NULL;
+    const char *database=NULL, *user=NULL, *password=NULL;
+    const char *host=NULL, *sslmode=NULL;
     char port[16];
 
     static char *kwlist[] = {"dsn", "database", "host", "port",
@@ -166,7 +145,7 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
       PyOS_snprintf(port, 16, "%d", iport);
 
     if (dsn_static == NULL) {
-        size_t l = 45; /* len("dbname= user= password= host= port= sslmode=\0") */
+        size_t l = 46; /* len(" dbname= user= password= host= port= sslmode=\0") */
 
         if (database) l += strlen(database);
         if (host) l += strlen(host);
@@ -212,7 +191,6 @@ psyco_connect(PyObject *self, PyObject *args, PyObject *keywds)
       /* allocate connection, fill with errors and return it */
       if (factory == NULL) factory = (PyObject *)&connectionType;
       conn = PyObject_CallFunction(factory, "s", dsn);
-      if (conn) _psyco_connect_fill_exc((connectionObject*)conn);
     }
 
     goto cleanup;
@@ -330,7 +308,8 @@ psyco_adapters_init(PyObject *mod)
 #endif
 
 #ifdef HAVE_DECIMAL
-    microprotocols_add((PyTypeObject*)decimalType, NULL, (PyObject*)&asisType);
+    microprotocols_add((PyTypeObject*)psyco_GetDecimalType(),
+                       NULL, (PyObject*)&asisType);
 #endif
 }
 
@@ -429,13 +408,16 @@ static void psyco_encodings_fill(PyObject *dict)
 PyObject *Error, *Warning, *InterfaceError, *DatabaseError,
     *InternalError, *OperationalError, *ProgrammingError,
     *IntegrityError, *DataError, *NotSupportedError;
+#ifdef PSYCOPG_EXTENSIONS
+PyObject *QueryCanceledError, *TransactionRollbackError;
+#endif
 
 /* mapping between exception names and their PyObject */
 static struct {
     char *name;
     PyObject **exc;
     PyObject **base;
-    char *docstr;
+    const char *docstr;
 } exctable[] = {
     { "psycopg2.Error", &Error, 0, Error_doc },
     { "psycopg2.Warning", &Warning, 0, Warning_doc },
@@ -451,6 +433,13 @@ static struct {
     { "psycopg2.DataError", &DataError, &DatabaseError, DataError_doc },
     { "psycopg2.NotSupportedError", &NotSupportedError, &DatabaseError,
         NotSupportedError_doc },
+#ifdef PSYCOPG_EXTENSIONS
+    { "psycopg2.extensions.QueryCanceledError", &QueryCanceledError,
+      &OperationalError, OperationalError_doc },
+    { "psycopg2.extensions.TransactionRollbackError",
+      &TransactionRollbackError, &OperationalError,
+      TransactionRollbackError_doc },
+#endif
     {NULL}  /* Sentinel */
 };
 
@@ -481,6 +470,13 @@ psyco_errors_init(void)
 
         *exctable[i].exc = PyErr_NewException(exctable[i].name, base, dict);
     }
+
+    /* Make pgerror, pgcode and cursor default to None on psycopg
+       error objects.  This simplifies error handling code that checks
+       these attributes. */
+    PyObject_SetAttrString(Error, "pgerror", Py_None);
+    PyObject_SetAttrString(Error, "pgcode", Py_None);
+    PyObject_SetAttrString(Error, "cursor", Py_None);
 }
 
 void
@@ -496,6 +492,11 @@ psyco_errors_fill(PyObject *dict)
     PyDict_SetItemString(dict, "IntegrityError", IntegrityError);
     PyDict_SetItemString(dict, "DataError", DataError);
     PyDict_SetItemString(dict, "NotSupportedError", NotSupportedError);
+#ifdef PSYCOPG_EXTENSIONS
+    PyDict_SetItemString(dict, "QueryCanceledError", QueryCanceledError);
+    PyDict_SetItemString(dict, "TransactionRollbackError",
+                         TransactionRollbackError);
+#endif
 }
 
 void
@@ -511,6 +512,11 @@ psyco_errors_set(PyObject *type)
     PyObject_SetAttrString(type, "IntegrityError", IntegrityError);
     PyObject_SetAttrString(type, "DataError", DataError);
     PyObject_SetAttrString(type, "NotSupportedError", NotSupportedError);
+#ifdef PSYCOPG_EXTENSIONS
+    PyObject_SetAttrString(type, "QueryCanceledError", QueryCanceledError);
+    PyObject_SetAttrString(type, "TransactionRollbackError",
+                           TransactionRollbackError);
+#endif
 }
 
 /* psyco_error_new
@@ -518,8 +524,8 @@ psyco_errors_set(PyObject *type)
    Create a new error of the given type with extra attributes. */
 
 void
-psyco_set_error(PyObject *exc, PyObject *curs, char *msg,
-                 char *pgerror, char *pgcode)
+psyco_set_error(PyObject *exc, PyObject *curs, const char *msg,
+                const char *pgerror, const char *pgcode)
 {
     PyObject *t;
 
@@ -528,50 +534,92 @@ psyco_set_error(PyObject *exc, PyObject *curs, char *msg,
     if (err) {
         if (pgerror) {
             t = PyString_FromString(pgerror);
+            PyObject_SetAttrString(err, "pgerror", t);
+            Py_DECREF(t);
         }
-        else {
-            t = Py_None ; Py_INCREF(t);
-        }
-        PyObject_SetAttrString(err, "pgerror", t);
-        Py_DECREF(t);
 
         if (pgcode) {
             t = PyString_FromString(pgcode);
+            PyObject_SetAttrString(err, "pgcode", t);
+            Py_DECREF(t);
         }
-        else {
-            t = Py_None ; Py_INCREF(t);
-        }
-        PyObject_SetAttrString(err, "pgcode", t);
-        Py_DECREF(t);
 
         if (curs)
             PyObject_SetAttrString(err, "cursor", curs);
-        else
-            PyObject_SetAttrString(err, "cursor", Py_None);
 
         PyErr_SetObject(exc, err);
     Py_DECREF(err);
     }
 }
 
-/* psyco_decimal_init
 
-   Initialize the module's pointer to the decimal type. */
-
-void
-psyco_decimal_init(void)
+/* Return nonzero if the current one is the main interpreter */
+static int
+psyco_is_main_interp(void)
 {
+    static PyInterpreterState *main_interp = NULL;  /* Cached reference */
+    PyInterpreterState *interp;
+
+    if (main_interp) {
+        return (main_interp == PyThreadState_Get()->interp);
+    }
+
+    /* No cached value: cache the proper value and try again. */
+    interp = PyInterpreterState_Head();
+    while (interp->next)
+        interp = interp->next;
+
+    main_interp = interp;
+    assert (main_interp);
+    return psyco_is_main_interp();
+}
+
+
+/* psyco_GetDecimalType
+
+   Return a new reference to the adapter for decimal type.
+
+   If decimals should be used but the module import fails, fall back on
+   the float type.
+
+    If decimals are not to be used, return NULL.
+   */
+
+PyObject *
+psyco_GetDecimalType(void)
+{
+    PyObject *decimalType = NULL;
+    static PyObject *cachedType = NULL;
+
 #ifdef HAVE_DECIMAL
     PyObject *decimal = PyImport_ImportModule("decimal");
+
+    /* Use the cached object if running from the main interpreter. */
+    int can_cache = psyco_is_main_interp();
+    if (can_cache && cachedType) {
+        Py_INCREF(cachedType);
+        return cachedType;
+    }
+
+    /* Get a new reference to the Decimal type. */
     if (decimal) {
         decimalType = PyObject_GetAttrString(decimal, "Decimal");
+        Py_DECREF(decimal);
     }
     else {
         PyErr_Clear();
         decimalType = (PyObject *)&PyFloat_Type;
         Py_INCREF(decimalType);
     }
-#endif
+
+    /* Store the object from future uses. */
+    if (can_cache && !cachedType) {
+        cachedType = decimalType;
+    }
+
+#endif /* HAVE_DECIMAL */
+
+    return decimalType;
 }
 
 
@@ -644,6 +692,11 @@ init_psycopg(void)
     PyObject *module, *dict;
     PyObject *c_api_object;
 
+#ifdef PSYCOPG_DEBUG
+    if (getenv("PSYCOPG_DEBUG"))
+      psycopg_debug_enabled = 1;
+#endif
+
     Dprintf("initpsycopg: initializing psycopg %s", PSYCOPG_VERSION);
 
     /* initialize all the new types and then the module */
@@ -667,6 +720,11 @@ init_psycopg(void)
     if (PyType_Ready(&listType) == -1) return;
     if (PyType_Ready(&chunkType) == -1) return;
 
+#ifdef PSYCOPG_EXTENSIONS
+    lobjectType.ob_type    = &PyType_Type;
+    if (PyType_Ready(&lobjectType) == -1) return;
+#endif
+
 #ifdef HAVE_PYBOOL
     pbooleanType.ob_type   = &PyType_Type;
     if (PyType_Ready(&pbooleanType) == -1) return;
@@ -730,7 +788,6 @@ init_psycopg(void)
     /* other mixed initializations of module-level variables */
     psycoEncodings = PyDict_New();
     psyco_encodings_fill(psycoEncodings);
-    psyco_decimal_init();
 
     /* set some module's parameters */
     PyModule_AddStringConstant(module, "__version__", PSYCOPG_VERSION);
@@ -743,6 +800,9 @@ init_psycopg(void)
     PyModule_AddObject(module, "connection", (PyObject*)&connectionType);
     PyModule_AddObject(module, "cursor", (PyObject*)&cursorType);
     PyModule_AddObject(module, "ISQLQuote", (PyObject*)&isqlquoteType);
+#ifdef PSYCOPG_EXTENSIONS
+    PyModule_AddObject(module, "lobject", (PyObject*)&lobjectType);
+#endif
 
     /* encodings dictionary in module dictionary */
     PyModule_AddObject(module, "encodings", psycoEncodings);
@@ -769,6 +829,10 @@ init_psycopg(void)
     listType.tp_alloc = PyType_GenericAlloc;
     chunkType.tp_alloc = PyType_GenericAlloc;
 
+#ifdef PSYCOPG_EXTENSIONS
+    lobjectType.tp_alloc = PyType_GenericAlloc;
+#endif
+
 #ifdef HAVE_PYDATETIME
     pydatetimeType.tp_alloc = PyType_GenericAlloc;
 #endif
