@@ -3,6 +3,18 @@ import os
 import unittest
 
 dbname = os.environ.get('PSYCOPG2_TESTDB', 'psycopg2_test')
+dbhost = os.environ.get('PSYCOPG2_TESTDB_HOST', None)
+dbport = os.environ.get('PSYCOPG2_TESTDB_PORT', None)
+dbuser = os.environ.get('PSYCOPG2_TESTDB_USER', None)
+
+# Construct a DSN to connect to the test database:
+dsn = 'dbname=%s' % dbname
+if dbhost is not None:
+    dsn += ' host=%s' % dbhost
+if dbport is not None:
+    dsn += ' port=%s' % dbport
+if dbuser is not None:
+    dsn += ' user=%s' % dbuser
 
 import bugX000
 import extras_dictcursor
@@ -12,6 +24,7 @@ import test_quote
 import test_connection
 import test_transaction
 import types_basic
+import test_lobject
 
 def test_suite():
     suite = unittest.TestSuite()
@@ -23,6 +36,7 @@ def test_suite():
     suite.addTest(test_connection.test_suite())
     suite.addTest(test_transaction.test_suite())
     suite.addTest(types_basic.test_suite())
+    suite.addTest(test_lobject.test_suite())
     return suite
 
 if __name__ == '__main__':
